* -------------------------------------------------------------- tsplot_sub
      subroutine tsplot_sub(ts2plot, ndimen1, ndimen2, 
     :          sps, npts, ntraces,
     :          t_strt_in, t_plot_in, pp_cm,
     :          f_ps, ts_name, type_c)
   
* Dates: 05/31/09 - Written by D. Boore, based on smctsplt
*        06/25/09 - Use get_date, get_time to obtain the date and time.
*        11/03/09 - Added "close(nu_rs)"
!        06/03/10 - Increase max number of plotted traces to 22
!        02/03/12 - Increase max number of plotted traces to 23
 
      parameter (ntraces_max_dimen=31)
      integer npts(*)
      real sps(*)

      character f_ps*(*), ts_name(*)*(*), type_c(*)*(*)

      character stacode*4, prnt_f_name_l_r*10
      character*30 stname
      character*5 comp, zero_line
      character*80 eqname
      character label*110, stem*40, date_c*10, time_c*11
      
      character f_ctl*80, buf*80, f_smc(ntraces_max_dimen)*80, 
     :          avdu_c*3, buf_path*120, buf_work*120
      logical f_ctl_exist, f_smc_exist

      real seis(:), ts2plot(ndimen1,ndimen2)
      allocatable :: seis

      parameter (XAXLEN=16.)	! length of x (time) axis, cm

      dpi_in = 200
      
      sign_ts = +1.0

      zero_line = ' '
      zero_line = 'y'
      call trim_c(zero_line,nc_zero_line)
      call upstr(zero_line)
      
!      ntraces_max = 22
      ntraces_max = min0(ntraces, 23)
      
      gray_scale = 0.5      
      zero_line_width = 1.0
      
      prnt_f_name_l_r(1:1) = 'L'
      
      if (ntraces*(pp_cm+0.5) .gt. 21.0) then
        pp_cm = 21.0/float(ntraces_max) - 0.5
        write(*,'(a,1x,f5.2)') ' Resetting pp_cm to ', pp_cm
      end if 
      
      t_plot = t_plot_in
      
* Compute tmax for the set
      tmax = 0.0
      do i = 1, ntraces 
         if (float(npts(i)-1)/sps(i) .gt. tmax) 
     :        tmax = float(npts(i)-1)/sps(i)
      end do

      if(t_plot .gt. tmax) then
        t_plot = tmax
      end if

* See if have to reset dpi to make sure that have at least two samples per pixel
*   First find minimum sample rate for this group of traces:

      dpi = dpi_in
      sps_min = 10000.0
      do itrace = 1, ntraces
        if (sps(itrace) .lt. sps_min) then
          sps_min = sps(itrace)
        end if
      end do
      
*   Then check ptppx and reset dpi if needed:
      ptppx = (2.54/dpi) *(sps_min * t_plot)/xaxlen

      dpc = dpi/2.54                   ! dots per cm
      lentic = nint(0.2*dpc)
      icm = nint(dpc)
      ipp_cmd2 = nint(0.5*pp_cm*dpc)
      ifntscale=nint(84.*dpi/600.)

* Construct name of ps file and open it:
      call get_lun(nu_ps)
      call trim_c(f_ps,nc_f_ps)
      open(unit=nu_ps,file=f_ps(1:nc_f_ps),status='unknown')

      do itrace = 1, ntraces
      
        allocate( seis(0:npts(itrace)) )
        
        seis = 0.0
        
        do i= 1, npts(itrace)
          seis(i) = ts2plot(i,itrace)
        end do

        tskip = t_strt_in ! not used in this version
        tlength = 2000.0
        
        srate = sps(itrace)
        ndata = npts(itrace) 
 
        if (float(ndata-1)/srate .gt. t_plot) then
          npts4plot = int(t_plot*srate) + 1
        else
          npts4plot = ndata
        end if

        write(*,'(a)') 
     :      ' itrace, srate, ndata, npts4plot, t_plot = '
        write(*,*) 
     :        itrace, srate, ndata, npts4plot, t_plot

        do i = 1, npts4plot
          seis(i) = sign_ts * seis(i)
        end do

        call mnmax(seis,1,npts4plot,1,smin,smax)

        if (itrace .eq. 1) then
        
* Get date, time:
          call get_date(date_c)
          call get_time(time_c)
          
* Write definitions etc to ps file:
          call header(nu_ps,DPI)

* Set up time axis, ticks, label at bottom:
          ix0 = 0
          iy0 = nint(-(ntraces*(pp_cm+0.5)+0.5)*DPC)
          write(nu_ps,'(2i6," M",i6," 0 r")') 
     :                               ix0, iy0, nint(XAXLEN*DPC)

          if (t_plot .le. 10.0) then
            tick_dt = 1.0
          else if (t_plot .le. 50.0) then
            tick_dt = 5.0
          else if (t_plot .le. 100.0) then
            tick_dt = 10.0
          else if (t_plot .le. 500.0) then
            tick_dt = 50.0
          else if (t_plot .le. 1000.0) then
            tick_dt = 100.0
          else if (t_plot .le. 5000.0) then
            tick_dt = 500.0
          else
            tick_dt = 1000.0
          end if

          ntics = t_plot/tick_dt + 1
          do itic = 1, ntics
            ix = nint((itic-1)*tick_dt*XAXLEN*DPC/t_plot)
            write(nu_ps,'(2i6," M 0",i4," r")') ix, iy0, lentic
            write(nu_ps,'("0 (",i5,") stringwidth pop 2 div neg",i5,
     :          " v (",i5,") show")')
     :          nint((itic-1)*tick_dt+t_strt_in), 
     :         -lentic-nint(0.1*DPC)-ifntscale, 
     :          nint((itic-1)*tick_dt+t_strt_in)
          end do

          write(nu_ps,'("S")')
          label = ' '
      	  write(label,'("TIME, s   (Duration =",f7.2,")")')
     :      t_plot
          write(nu_ps,'(i6," (",a,") stringwidth pop 2 div sub",i6,
     :      " M (",a,") show")')
     :      nint(0.5*XAXLEN*DPC), label(:lastc(label)),
     :      iy0 - nint(1.*DPC),
     :      label(:lastc(label))
     
          print *,' '
          print *, 'Do not be concerned about a "File Not Found"'//
     :             ' message. '
          print *, 'It appears because the call to Header is looking'//
     :             ' for smctsplt.ctl; if it is not in the folder '
          print *, '(as can happen if filt_plot is being run),' 
          print *, 'the message is returned in a call to System.'
          print *, 'There is no variable in System that allows trapping'
          print *, 'of the message.'
          print *, 'The proper solution would be to pass the command' 
          print *, 'string into Header, but it will be different for'
          print *, 'any program that calls Header (e.g., smctsplt and'
          print *, 'filt_plot at present).  Even though an error'
          print *, 'message is produced, the correct path name is found'
          print *, 'and included in the ps plot.'
          print *,' '
 
          buf_path = ' '
          call get_path_from_system_call(buf_path, nc_buf_path)
          
          label = ' '
      	  label = 'Path:'//buf_path(14:nc_buf_path)
      	  print *, ' buf_path(14:nc_buf_path) = ', 
     :               buf_path(14:nc_buf_path) 
          
          write(nu_ps,'(i6," (",a,") stringwidth pop 2 div sub",i6,
     :      " M (",a,") show")')
     :      nint(0.5*XAXLEN*DPC), label(:lastc(label)),
     :      iy0 - nint(1.5*DPC),
     :      label(:lastc(label))
 
          label = ' '
      	  label = 'File:'//f_ps(1:nc_f_ps)//' -- Date:'//date_c//
     :            ' -- Time:'//time_c
          write(nu_ps,'(i6," (",a,") stringwidth pop 2 div sub",i6,
     :      " M (",a,") show")')
     :      nint(0.5*XAXLEN*DPC), label(:lastc(label)),
     :      iy0 - nint(2.0*DPC),
     :      label(:lastc(label))
        endif  ! end of writing of labels, now proceed to writing the time series

        iy0 = -nint(DPC*( float(itrace-1)*(pp_cm+0.5) + 0.5*pp_cm ))
      	  
        write(nu_ps,'(2i6," M")') 0, iy0
        
        call trim_c(ts_name(itrace), nc_ts_name)

* Write time series file name:
        if (prnt_f_name_l_r(1:1) .eq. 'L') then
        
          write(nu_ps, '(2i6,a)')
     :     nint(2.0*lentic), 
     :     ipp_cmd2 - nint(1.5*real(lentic)),
     :     ' v ('//ts_name(itrace)(1:nc_ts_name)// 
     :     ') show'
        else if (prnt_f_name_l_r(1:1) .eq. 'R') then
          write(nu_ps, '(2i6,a)')
     :     nint(xaxlen*dpc)- nc_ts_name*ifntscale - lentic/2, 
     :     ipp_cmd2 - nint(1.5*real(lentic)),
     :     ' v ('//ts_name(itrace)(1:nc_ts_name)//
     :     ') show'
        else
          write(*,'(a)') ' Error!!!, prnt_f_name_l_r = '//
     :        prnt_f_name_l_r(1:1)//'; invalid; quitting'
          stop
        end if


* Write type of motion:
        avdu_c = ' '
        avdu_c = type_c(itrace)(1:3)
        call upstr(avdu_c(1:3))
        ix = nint(DPC*(-1.5))
        write(nu_ps,'(2i6," M 90 rotate")') ix, iy0
        write(nu_ps,
     :        '("(",a3,") stringwidth pop 2 div ",
     :          "neg 0 v ",
     :		"(",a3,") show -90 rotate")') AVDU_C, AVDU_C

* Write y-axis line and ticks on left side:
        write(nu_ps,'(2i6," M")') 0, iy0
        
        if (smin .lt. 0.0 .and. smax .gt. 0.0) then

* Write y-axis line and ticks on left side:
 
* y location of zero line, relative to top of left ordinate axis:
          i2zero = - int((smax/(smax-smin))*float(2*ipp_cmd2))
          
          i2bottom = - 2*ipp_cmd2 - i2zero
          write(nu_ps,'(2i5," v",
     :                  2i5," r",
     :                  2i5," r",
     :                  2i5," r",
     :                  2i5," r",
     :                   2i5," r",
     :                  2i5," r S")')
     :       -lentic, ipp_cmd2, 
     :        lentic, 0,
     :             0, i2zero,
     :       -lentic, 0,
     :        lentic, 0,
     :             0, i2bottom, 
     :       -lentic, 0
          
            
           if (zero_line(1:1) .eq. 'Y') then

* Write gray zero line:
* y location of zero line, relative to current origin iy0
             write(nu_ps,'(a)') '%'
             write(nu_ps,'(f5.2," newpen")') zero_line_width
             write(nu_ps,'(f5.2," setgray")') gray_scale
           
             write(nu_ps,'(2i6," M")') 0, iy0
             write(nu_ps,'(2i6," v")') 0, ipp_cmd2 ! do this rather than the more
! direct use of " 0 izero_from_iy0 v" because integer truncation caused a one pixel
! difference in the location of the tick mark and the zero line
             write(nu_ps,'(2i6," v")') 0, i2zero
             write(nu_ps,'(2i5, " r S")') nint(XAXLEN*DPC),0

             write(nu_ps,'("0.0 setgray")') 
             write(nu_ps,'("0.33 newpen")')      
             write(nu_ps,'(a)') '%'
 
           end if

 
         
        else  
        
          write(nu_ps,'(2i5," v",2i5," r",2i5," r",2i5," r S")')
     :     -lentic,ipp_cmd2, lentic,0, 0,-2*ipp_cmd2, -lentic,0
     
        end if

* Label y axis with max, min amplitudes:
        if (abs(smax) .ge. 0.1) then
          write(nu_ps,'(i4,"(",f7.1," ) stringwidth pop sub",
     :        i6," M (",f7.1,") show")')
     :        -lentic, smax, iy0+ipp_cmd2-ifntscale/4, smax
        else if (abs(smax) .ge. 0.01) then
          write(nu_ps,'(i4,"(",f7.2," ) stringwidth pop sub",
     :        i6," M (",f7.2,") show")')
     :        -lentic, smax, iy0+ipp_cmd2-ifntscale/4, smax
        else if (abs(smax) .ge. 0.001) then
          write(nu_ps,'(i4,"(",f7.3," ) stringwidth pop sub",
     :        i6," M (",f7.3,") show")')
     :        -lentic, smax, iy0+ipp_cmd2-ifntscale/4, smax
        else
          write(nu_ps,'(i4,"(",f7.4," ) stringwidth pop sub",
     :        i6," M (",f7.4,") show")')
     :        -lentic, smax, iy0+ipp_cmd2-ifntscale/4, smax
        endif

        if (abs(smin) .ge. 0.1) then
          write(nu_ps,'(i4,"(",f7.1," ) stringwidth pop sub",
     :        i6," M (",f7.1,") show")')
     :        -lentic, smin, iy0-ipp_cmd2-ifntscale/4, smin
        else if (abs(smin) .ge. 0.01) then
          write(nu_ps,'(i4,"(",f7.2," ) stringwidth pop sub",
     :        i6," M (",f7.2,") show")')
     :        -lentic, smin, iy0-ipp_cmd2-ifntscale/4, smin
        else if (abs(smin) .ge. 0.001) then
          write(nu_ps,'(i4,"(",f7.3," ) stringwidth pop sub",
     :        i6," M (",f7.3,") show")')
     :        -lentic, smin, iy0-ipp_cmd2-ifntscale/4, smin
        else
          write(nu_ps,'(i4,"(",f7.4," ) stringwidth pop sub",
     :        i6," M (",f7.4,") show")')
     :        -lentic, smin, iy0-ipp_cmd2-ifntscale/4, smin
        endif

        if (smin .lt. 0.0 .and. smax .gt. 0.0) then
          
          idown1 = - int((smax/(smax-smin))*float(2*ipp_cmd2))
          idown2 = - 2*ipp_cmd2 - idown1
          zero_label = 0.0
          write(nu_ps,'(i4,"(",f7.1," ) stringwidth pop sub",
     :        i6," M (",f7.1,") show")')
     :        -lentic, zero_label, 
     :        iy0+ipp_cmd2+idown1-ifntscale/4, zero_label
     
        end if



* Write y-axis line and ticks on right side:
        write(nu_ps,'(2i6," M")') nint(XAXLEN*DPC), iy0

        if (smin .lt. 0.0 .and. smax .gt. 0.0) then
          
          idown1 = - int((smax/(smax-smin))*float(2*ipp_cmd2))
          idown2 = - 2*ipp_cmd2 - idown1
          write(nu_ps,'(2i5," v",2i5," r",
     :                  2i5," r",
     :                  2i5," r",
     :                  2i5," r",
     :                  2i5," r",
     :                  2i5," r S")')
     :     lentic,ipp_cmd2, -lentic,0,
     :          0, idown1,
     :     lentic, 0,
     :    -lentic, 0,
     :          0,idown2, 
     :     lentic,0


        else
        
          write(nu_ps,'(2i5," v",2i5," r",2i5," r",2i5," r S")')
     :      lentic,ipp_cmd2, -lentic,0, 0,-2*ipp_cmd2, lentic,0
     
        end if


* Code to draw bars between min and max values of all data points within a
* pixel

! ratio of plotted duration to t_plot
        xlencm = ((float(npts4plot)/srate)/t_plot)*xaxlen 
        ptppx = float(npts4plot)/(xlencm*dpc)
        xlast = 0
        xend = ptppx
        ipixel = 1

        yscal = (pp_cm/(smax-smin))* dpc
        xscal = XLENcm*DPC/npts4plot
        scenter = 0.5*(smax+smin)

        i = 0
        ymin = seis(1)
        ymax = seis(1)

        do while (i+1 .lt. npts4plot)  ! change made 1/02/03
          i = i + 1
          x = float(i)
          if (x .lt. xend) then
* Find min, max y value for the x values within one x pixel:
            if (seis(i) .gt. ymax) then
              ymax = seis(i)
            else if (seis(i) .lt. ymin) then
              ymin = seis(i)
            endif             
          else                              ! stepped out of this x-pixel range
* Write bar:
            ix = nint( 0.5*(xlast+xend) * xscal )
*            ix = nint( xlast * xscal )
            iymin = nint( (ymin - scenter) * yscal )
            iymax = nint( (ymax - scenter) * yscal )
            write(nu_ps,'( 2i6, " M", 2i6, " r")')
     :        ix,iy0 + iymin, 0,iymax - iymin

* Reset for next x-pixel:            
            xlast = xend
            ipixel = ipixel + 1
            xend = ipixel * ptppx
            ymin = seis(i-1)
            ymax = ymin
            if (seis(i) .gt. ymax) then
              ymax = seis(i)
            else if (seis(i) .lt. ymin) then
              ymin = seis(i)
            endif             
          end if
        end do

        ix = nint(0.5*(xlast+xend)*xscal)
        iymin = nint( (ymin - scenter) * yscal )
        iymax = nint( (ymax - scenter) * yscal )
        write(nu_ps,'( 2i6, " M", 2i6, " r")')
     :        ix,iy0 + iymin, 0,iymax - iymin


        write(nu_ps,'("S")')

        deallocate(seis)

      enddo  ! loop over traces for this group

      write(*,'(2a)') ' Writing output file = ', f_ps(1:nc_f_ps)
      write(nu_ps,'("showpage")')
      
      close(nu_ps)

      return
      
      end
* -------------------------------------------------------------- smctsplt

* -------------------------------------------------------------- begin header
	subroutine header(luno,dpi)
* Dates: 09/03/00 - The size in the font spec was changed by D. Boore

	integer luno	! logical output unit
	real*4 dpi	! plotter dpi

	idpi = nint(dpi)
        ifntscale=nint(84.*dpi/600.)
        s_line = dpi/600.0
        
	write(luno,'(
     :      "%!"/
     :      "72",i6," div dup scale"/
     :      "1 setlinecap"/
     :      "1 setlinejoin"/
     :      "userdict /picstr 2 string put"/
     :      "/M /moveto load def"/
     :      "/v /rmoveto load def"/
     :      "/L /lineto load def"/
     :      "/r /rlineto load def"/
     :      "/S /stroke load def"/
     :      "/A /setgray load def"/
     :      "/C /setrgbcolor load def"/
     :      "/newpen /setlinewidth load def"/
     :      "%"/
     :      "% ------------------------------------------"/
     :       1x, f5.2, " newpen")') idpi, s_line
*     :      "1 newpen")') idpi
	write(luno,'(
     :      "1.25",i5," mul 10.0",i5," mul translate"/
     :      "/Helvetica findfont ",i2," scalefont setfont"/   
     :      "%")')  idpi, idpi, ifntscale

	return
	end
* -------------------------------------------------------------- end header


c -------------------------------------------------------------------  locase
	subroutine locase(string)

c  convert ascii string to lower case

	character*(*) string

        ioff = ichar('a') - ichar('A')

	i = 0
	do while (i .lt. len(string))
	  i = i + 1
	  if (string(i:i) .ge. 'A' .and. string(i:i) .le. 'Z')
     *		string(i:i) = char(ichar(string(i:i)) + ioff)
	enddo

	return

	end
c -------------------------------------------------------------------  locase

c -------------------------------------------------------------------  upcase
	subroutine upcase(string)

c  convert ascii string to upper case

	character*(*) string

	ioff = ichar('A') - ichar('a')

	i = 0
	do while (i .lt. len(string))
	  i = i + 1
	  if (string(i:i) .ge. 'a' .and. string(i:i) .le. 'z')
     *		string(i:i) = char(ichar(string(i:i)) + ioff)
	enddo

	return

	end
c -------------------------------------------------------------------  upcase

c -------------------------------------------------------------------  right
	subroutine right(string)

c  right-justify characters in ascii string

	character*(*) string

	lens = len(string)

	ie = lens
	do while (ie .gt. 0 .and. string(ie:ie) .eq. ' ')
	  ie = ie - 1
	enddo

	if (ie .eq. 0) return

	ishift = lens - ie
	i = lens
	j = i - ishift
	do while (i .gt. lens-ie)
	  string(i:i) = string(j:j)
	  i = i - 1
	  j = j - 1
	enddo
	do while (i .gt. 0)
	  string(i:i) = ' '
	  i = i - 1
	enddo

	return

	end
c -------------------------------------------------------------------  right

c -------------------------------------------------------------------  left
	subroutine left(string)

c  left-justify characters in ascii string

	character*(*) string

	lens = len(string)

	ib = 1
	do while (ib .le. lens .and. string(ib:ib) .eq. ' ')
	  ib = ib + 1
	enddo

	if (ib .eq. 1 .or. ib .gt. lens) return

	i = 1
	j = ib
	do while (i .lt. lens-ib+2)
	  string(i:i) = string(j:j)
	  i = i + 1
	  j = j + 1
	enddo
	do while (i .le. lens)
	  string(i:i) = ' '
	  i = i + 1
	enddo

	return

	end
c -------------------------------------------------------------------  left

c -------------------------------------------------------------------  lastc
	integer function lastc(string)

c  return position of last non-blank character in an ascii string
c  a value of 0 is returned for all blanks

	character*(*) string

	lastc = len(string)

	do while (lastc .gt. 0 .and. (string(lastc:lastc) .eq. ' ' 
     *		.or. string(lastc:lastc) .eq. '\0'))
	  lastc = lastc - 1
	enddo

	return

	end
c -------------------------------------------------------------------  lastc

c -------------------------------------------------------------------  iargs
	integer function iargs(string)

c  return the number of arguments in an ascii string.  arguments are delimited
c  by 'white' space (blanks or tabs) or by using commas.  single (') or double
c  (") quotes can be used to delimit contingous substrings that have embedded 
c  blanks or tabs (e.g. 'this example'); to preserve a single-quote character 
c  within a substring, use a double single-quote, (e.g. 'can''t' will be parsed 
c  as can't).

c  -1 is returned if an error is detected (e.g. format, missing quote delimiter)

	character*1 tab, quote, delim
	parameter (tab = char(9))
	parameter (quote = char(39))

	character*(*) string

	iargs = 0
	lens = len(string)
	ic = 0
	iarg = 0
	delim = ' '

c  -- begin main loop
	
	do while (ic .le. lens)

	  ib = ic + 1

c -- skip initial white space before argument

	  do while (ib .le. lens .and.
     *	 	(string(ib:ib) .eq. ' '	.or. string(ib:ib) .eq. tab))
	    ib = ib + 1
	  enddo

	  if (ib .gt. lens) then
	    if (delim .eq. ',') iargs = iargs + 1
	    return
	  endif

c -- implied argument (e.g. ',,').  not the same as null (ascii 0)!

	  if (string(ib:ib) .eq. ',') then
	    ic = ib

c -- quoted string using single-quote delimiter

	  else if (string(ib:ib) .eq. quote) then

	    ib = ib + 1
	    if (ib .gt. lens) go to 90
	    ic = ib
20	    do while (ic .le. lens .and.  string(ic:ic) .ne. quote)
	      ic = ic + 1
	    enddo
	    if (ic .gt. lens) go to 90
	    if (ic .lt. lens .and. string(ic+1:ic+1) .eq. quote) then
	      ic = ic + 2
	      if (ic .gt. lens) go to 90
	      go to 20
	    endif

	    if (ic .lt. lens) then
	      ic = ic + 1
              if (string(ic:ic) .ne. ' ' .and. string(ic:ic) .ne. tab
     *         .and. string(ic:ic) .ne. ',') go to 90
	    else
	      ic = ic + 1
	    endif
	
c -- quoted string using double-quote delimiter

	  else if (string(ib:ib) .eq. '"') then

	    ib = ib + 1
	    if (ib .gt. lens) go to 90
	    ic = ib

	    do while (ic .le. lens .and.  string(ic:ic) .ne. '"')
	      ic = ic + 1
	    enddo

	    if (ic .lt. lens) then
	      ic = ic + 1
              if (string(ic:ic) .ne. ' ' .and. string(ic:ic) .ne. tab
     *          .and. string(ic:ic) .ne. ',') go to 90

	    else if (ic .gt. lens) then
	      go to 90

	    else
	      ic = ic + 1
	    endif

c -- other string

	  else

	    ic = ib + 1
	    do while (ic .le. lens .and. string(ic:ic) .ne. ',' .and.
     *		string(ic:ic) .ne. ' ' .and. string(ic:ic) .ne. tab)
	      ic = ic + 1
	    enddo

	  endif

c -- found next argument

	  iargs = iargs + 1

c -- skip white space before delimiting comma or start of next argument

	  do while (ic .le. lens .and. (string(ic:ic) .eq. ' ' .or. 
     *		string(ic:ic) .eq. tab))
	    ic = ic + 1
	  enddo

	  if (ic .gt. lens) return

	  if (string(ic:ic) .ne. ',') ic = ic - 1

	  delim = string(ic:ic)

	enddo

	return

c -- error detected

90	iargs = -1

	return

	end
c -------------------------------------------------------------------  iargs

c -------------------------------------------------------------------  getstr
	subroutine getstr (narg, string, substr, nchar)

c  return the narg'th argument in an ascii string.  arguments are delimited
c  by 'white' space (blanks or tabs) or by using commas.  single (') or double
c  (") quotes can be used to delimit contingous substrings that have embedded 
c  blanks or tabs (e.g. 'this example'); to preserve a single-quote character 
c  within a substring, use a double single-quote, (e.g. 'can''t' will be parsed 
c  as can't).

c  on return, nchar = -1 if an error is detected (e.g. format, missing quote
c			 delimiter, length of target argument exceeds size of
c			 substr).  
c		    =  0 if narg exceeds the number of arguments in the
c			 search string
c                   =  index of last non-blank character in the argument, unless
c			 the argument is all blank, in which case substr is
c			 all blank characters and nchar = length of substr

c  the function iargs returns the number of arguments in an ascii string

c  cds 08-04-93

	character*1 tab, quote, delim
	parameter (tab = char(9))
	parameter (quote = char(39))

	character*(*) string	! search string
	character*(*) substr	! sub-string with target argument

	substr = ' '
	nchar = -1
	if (narg .lt. 1) return

	lens = len(string)
	ic = 0
	ie = 0
	iarg = 0
	delim = ' '

c  -- begin main loop
	
	do while (iarg .lt. narg .and. ic .le. lens)

	  ib = ic + 1
	  nchar = -1

c -- skip initial white space before argument

	  do while (ib .le. lens .and.
     *	 	(string(ib:ib) .eq. ' '	.or. string(ib:ib) .eq. tab))
	    ib = ib + 1
	  enddo

	  if (ib .gt. lens) then
	    if (delim .eq. ',' .or. iarg .eq. 0) then
	      iarg = iarg + 1
	      if (iarg .eq. narg) nchar = len(substr)
	    endif
	    return
	  endif

c -- implied argument (e.g. ',,').  not the same as null (ascii 0)!

	  if (string(ib:ib) .eq. ',') then
	    ic = ib
	    nchar = len(substr)

c -- quoted string using single-quote delimiter

	  else if (string(ib:ib) .eq. quote) then

	    ib = ib + 1
	    if (ib .gt. lens) return
	    ic = ib
20	    do while (ic .le. lens .and.  string(ic:ic) .ne. quote)
	      ic = ic + 1
	    enddo
	    if (ic .gt. lens) return
	    if (ic .lt. lens .and. string(ic+1:ic+1) .eq. quote) then
	      ic = ic + 2
	      if (ic .gt. lens) return
	      go to 20
	    endif

	    ie = ic - 1

	    if (ic .lt. lens) then
	      ic = ic + 1
              if (string(ic:ic) .ne. ' ' .and. string(ic:ic) .ne. tab 
     *          .and. string(ic:ic) .ne. ',') return
	    else
	      ic = ic + 1
	    endif
	
c -- quoted string using double-quote delimiter

	  else if (string(ib:ib) .eq. '"') then

	    ib = ib + 1
	    if (ib .gt. lens) return
	    ic = ib

	    do while (ic .le. lens .and.  string(ic:ic) .ne. '"')
	      ic = ic + 1
	    enddo

	    ie = ic - 1

	    if (ic .lt. lens) then
	      ic = ic + 1
	      if (string(ic:ic) .ne. ' ' .and. string(ic:ic) .ne. tab
     *         .and. string(ic:ic) .ne. ',') return

	    else if (ic .gt. lens) then
	      return

	    else
	      ic = ic + 1
	    endif

c -- other string

	  else

	    ic = ib + 1
	    do while (ic .le. lens .and. string(ic:ic) .ne. ',' .and.
     *		string(ic:ic) .ne. ' ' .and. string(ic:ic) .ne. tab)
	      ic = ic + 1
	    enddo

	    ie = ic - 1

	  endif

c -- found end of argument

	  iarg = iarg + 1

c -- is this the target?

	  if (iarg .eq. narg) then

	    if (nchar .gt. -1) then
	      return
	    else
	      nchar = ie - ib + 1
	      if (nchar .gt. len(substr)) then
		nchar = -1
	      else
		substr = string(ib:ie)
	      endif
	      return
	    endif

	  endif

c -- skip white space before delimiting comma or start of next argument

	  do while (ic .le. lens .and. (string(ic:ic) .eq. ' ' .or. 
     *		string(ic:ic) .eq. tab))
	    ic = ic + 1
	  enddo

	  if (ic .gt. lens) return

	  if (string(ic:ic) .ne. ',') ic = ic - 1

	  delim = string(ic:ic)

	enddo

	end
c -------------------------------------------------------------------  getstr


